# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding model 'Campaign'
        db.create_table('mailchimp_campaign', (
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('sent_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('campaign_id', self.gf('django.db.models.fields.CharField')(max_length=50)),
        ))
        db.send_create_signal('mailchimp', ['Campaign'])

        # Adding model 'Reciever'
        db.create_table('mailchimp_reciever', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('campaign', self.gf('django.db.models.fields.related.ForeignKey')(related_name='recievers', to=orm['mailchimp.Campaign'])),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
        ))
        db.send_create_signal('mailchimp', ['Reciever'])
    
    
    def backwards(self, orm):
        
        # Deleting model 'Campaign'
        db.delete_table('mailchimp_campaign')

        # Deleting model 'Reciever'
        db.delete_table('mailchimp_reciever')
    
    
    models = {
        'mailchimp.campaign': {
            'Meta': {'object_name': 'Campaign'},
            'campaign_id': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sent_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'mailchimp.reciever': {
            'Meta': {'object_name': 'Reciever'},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'recievers'", 'to': "orm['mailchimp.Campaign']"}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['mailchimp']



from django.contrib import admin
from mailchimp.models import Campaign
from mailchimp.settings import VIEWS_OVERVIEW


class MailchimpAdmin(admin.ModelAdmin):
    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urlpatterns = patterns('',
            url(r'^$',
                VIEWS_OVERVIEW,
                name='mailchimp_campaign_changelist',
                kwargs={'page':'1'}),
        )
        return urlpatterns
    
    def has_add_permission(self, request):
        # disable the 'add' button
        return False
    
    def has_change_permission(self, request, obj=None):
        return request.user.has_perm('mailchimp.can_view')
        
    def has_delete_permission(self, request, obj=None):
        return False


admin.site.register(Campaign, MailchimpAdmin)


from django import template
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from mailchimp.utils import is_queued_or_sent

register = template.Library()


def mailchimp_send_for_object(context, object):
    is_sent = is_queued_or_sent(object)
    sent_date = None
    campaign_id = None
    if is_sent and hasattr(is_sent, 'sent_date'):
        sent_date = is_sent.sent_date
        campaign_id = is_sent.campaign_id
    if hasattr(object, 'mailchimp_allow_send'):
        objchck = object.mailchimp_allow_send
    else:
        objchck = lambda r: True
    request = context['request']
    return {
        'content_type': ContentType.objects.get_for_model(object).pk,
        'primary_key': object.pk,
        'allow': request.user.has_perm('mailchimp.can_send') and objchck(request),
        'is_sent': is_sent,
        'sent_date': sent_date,
        'campaign_id': campaign_id,
        'can_view': sent_date and request.user.has_perm('mailchimp.can_view'),
        'admin_prefix': settings.ADMIN_MEDIA_PREFIX,
        'can_test': bool(request.user.email),
    }
register.inclusion_tag('mailchimp/send_button.html', takes_context=True)(mailchimp_send_for_object)


from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib import messages 
from django.core.urlresolvers import reverse
from django.core.cache import cache
from django.contrib.contenttypes.models import ContentType
from django.utils import simplejson
from django.contrib.auth import logout
from django.contrib.messages import debug, info, success, warning, error, add_message
from django.http import (
    HttpResponse, HttpResponseForbidden, Http404, HttpResponseNotAllowed,
    HttpResponseRedirect, HttpResponsePermanentRedirect, HttpResponseNotModified,
    HttpResponseBadRequest, HttpResponseNotFound, HttpResponseGone,
    HttpResponseServerError
)
from mailchimp.settings import API_KEY, SECURE, REAL_CACHE, CACHE_TIMEOUT
import re
import warnings

class KeywordArguments(dict):
    def __getattr__(self, attr):
        return self[attr]


class Cache(object):
    def __init__(self, prefix=''):
        self._data = {}
        self._clear_lock = False
        self._prefix = prefix
        if REAL_CACHE:
            self._set = getattr(self, '_real_set')
            self._get = getattr(self, '_real_get')
            self._del = getattr(self, '_real_del')
        else:
            self._set = getattr(self, '_fake_set')
            self._get = getattr(self, '_fake_get')
            self._del = getattr(self, '_fake_del')
            

    def get(self, key, obj, *args, **kwargs):
        if self._clear_lock:
            self.flush(key)
            self._clear_lock = False
        value = self._get(key)
        if value is None:
            value = obj(*args, **kwargs) if callable(obj) else obj          
            self._set(key, value)
        return value
    
    def _real_set(self, key, value):
        cache.set(key, value, CACHE_TIMEOUT)
    
    def _real_get(self, key):
        return cache.get(key, None)
    
    def _real_del(self, key):
        cache.delete(key)
    
    def _fake_set(self, key, value):
        self._data[key] = value
    
    def _fake_get(self, key):
        return self._data.get(key, None)
    
    def _fake_del(self, key):
        if key in self._data:
            del self._data[key]
    
    def get_child_cache(self, key):
        return Cache('%s_%s_' % (self._prefix, key))
    
    def flush(self, *keys):
        for key in keys:
            if key in self._data:
                self._del(key)
            
    def lock(self):
        self._clear_lock = True

    def clear(self, call):
        self.lock()
        return call()


def wrap(base, parent, name, *baseargs, **basekwargs):
    def _wrapped(*args, **kwargs):
        fullargs = baseargs + args
        kwargs.update(basekwargs)
        return getattr(parent, '%s_%s' % (base, name))(*fullargs, **kwargs)
    return _wrapped


def build_dict(master, klass, data, key='id'):
    return  dict([(info[key], klass(master, info)) for info in data])

def _convert(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


class Bullet(object):
    def __init__(self, number, link, active):
        self.number = number
        self.link = link
        self.active = active


class Paginator(object):
    def __init__(self, objects, page, get_link, per_page=20, bullets=5):
        page = int(page)
        self.page = page
        self.get_link = get_link
        self.all_objects = objects
        self.objects_count = objects.count()
        per_page = per_page() if callable(per_page) else per_page
        self.pages_count = int(float(self.objects_count) / float(per_page)) + 1
        self.bullets_count = 5
        self.per_page = per_page
        self.start = (page - 1) * per_page
        self.end = page * per_page
        self.is_first = page == 1
        self.first_bullet = Bullet(1, self.get_link(1), False)
        self.is_last = page == self.pages_count
        self.last_bullet = Bullet(self.pages_count, self.get_link(self.pages_count), False)
        self.has_pages = self.pages_count != 1
        self._objects = None
        self._bullets = None
        
    @property
    def bullets(self):
        if self._bullets is None:
            pre = int(float(self.bullets_count) / 2)
            bullets = [Bullet(self.page, self.get_link(self.page), True)]
            diff = 0
            for i in range(1, pre + 1):
                this = self.page - i
                if this:
                    bullets.insert(0, Bullet(this, self.get_link(this), False))
                else:
                    diff = pre - this
                    break
            for i in range(1, pre + 1 + diff):
                this = self.page +  i
                if this <= self.pages_count:
                    bullets.append(Bullet(this, self.get_link(this), False))
                else:
                    break
            self._bullets = bullets
        return self._bullets
        
    @property
    def objects(self):
        if self._objects is None:
            self._objects = self.all_objects[self.start:self.end]
        return self._objects

    
class InternalRequest(object):
    def __init__(self, request, args, kwargs):
        self.request = request
        self.args = args
        self.kwargs = kwargs
        
    def contribute_to_class(self, cls):
        cls.request = self.request
        cls.args = self.args
        cls.kwargs = self.kwargs
    

class BaseView(object):
    """
    A base class to create class based views.
    
    It will automatically check allowed methods if a list of allowed methods are
    given. It also automatically tries to route to 'handle_`method`' methods if
    they're available. So if for example you define a 'handle_post' method and
    the request method is 'POST', this one will be called instead of 'handle'.
    
    For each request a new instance of this class will be created and it will get
    three attributes set: request, args and kwargs.
    """
    # A list of allowed methods (if empty any method will be allowed)
    allowed_methods = []
    # The template to use in the render_to_response helper
    template = 'base.html'
    # Only allow access to logged in users
    login_required = False
    # Only allow access to users with certain permissions
    required_permissions = []
    # Only allow access to superusers
    superuser_required = False
    # Response to send when request is automatically declined
    auto_decline_response = 'not_found'
    
    #===========================================================================
    # Dummy Attributes (DO NOT OVERWRITE)
    #=========================================================================== 
    request = None
    args = tuple()
    kwargs = {}
    
    #===========================================================================
    # Internal Methods
    #===========================================================================
    
    def __init__(self, *args, **kwargs):
        # Preserve args and kwargs
        self._initial_args = args
        self._initial_kwargs = kwargs

    @property
    def __name__(self):
        """
        INTERNAL: required by django
        """
        return self.get_view_name()
        
    def __call__(self, request, *args, **kwargs):
        """
        INTERNAL: Called by django when a request should be handled by this view.
        Creates a new instance of this class to sandbox 
        """
        if self.allowed_methods and request.method not in self.allowed_methods:
            return getattr(self, self.auto_decline_response)()
        if self.login_required and not request.user.is_authenticated():
            return getattr(self, self.auto_decline_response)()
        if self.superuser_required and not request.user.is_superuser:
            return getattr(self, self.auto_decline_response)()
        if self.required_permissions and not request.user.has_perms(self.required_permissions):
            return getattr(self, self.auto_decline_response)()
        handle_func_name = 'handle_%s' % request.method.lower()
        if not hasattr(self, handle_func_name):
            handle_func_name = 'handle'
        # Create a sandbox instance of this class to safely set the request, args and kwargs attributes
        sandbox = self.__class__(*self._initial_args, **self._initial_kwargs)
        sandbox.args = args
        sandbox.kwargs = kwargs
        sandbox.request = request
        return getattr(sandbox, handle_func_name)()
    
    #===========================================================================
    # Misc Helpers
    #===========================================================================
    
    def get_view_name(self):
        """
        Returns the name of this view
        """
        return self.__class__.__name__
    
    def get_template(self):
        return self.template
    
    def logout(self):
        logout(self.request)
        

    def get_page_link(self, page):
        return '%s?page=%s' % (self.request.path, page)
        
    def paginate(self, objects, page):
        return Paginator(objects, page, self.get_page_link, 20, 5)
    
    def reverse(self, view_name, *args, **kwargs):
        return reverse(view_name, args=args or (), kwargs=kwargs or {})
    
    #===========================================================================
    # Handlers
    #===========================================================================
    
    def handle(self):
        """
        Write your view logic here
        """
        pass
    
    #===========================================================================
    # Response Helpers
    #===========================================================================
    
    def not_allowed(self, data=''):
        return HttpResponseNotAllowed(data)
    
    def forbidden(self, data=''):
        return HttpResponseForbidden(data)
    
    def redirect(self, url):
        return HttpResponseRedirect(url)
    
    def named_redirect(self, viewname, urlconf=None, args=None, kwargs=None,
            prefix=None, current_app=None):
        return self.redirect(reverse(view, urlconf, args, kwargs, prefix, current_app))
    
    def permanent_redirect(self, url):
        return HttpResponsePermanentRedirect(url)
    
    def named_permanent_redirect(self, viewname, urlconf=None, args=None,
            kwargs=None, prefix=None, current_app=None):
        return self.permanent_redirect(reverse(view, urlconf, args, kwargs, prefix, current_app))
    
    def not_modified(self, data=''):
        return HttpResponseNotModified(data)
    
    def bad_request(self, data=''):
        return HttpResponseBadRequest(data)
    
    def not_found(self, data=''):
        return HttpResponseNotFound(data)
    
    def gone(self, data=''):
        return HttpResponseGone(data)
    
    def server_error(self, data=''):
        return HttpResponseServerError(data)
    
    def simplejson(self, data):
        return HttpResponse(simplejson.dumps(data), content_type='application/json')
    
    def response(self, data):
        return HttpResponse(data)
    
    def render_to_response(self, data, request_context=True):
        if request_context:
            return render_to_response(self.get_template(), data, RequestContext(self.request))
        return render_to_response(self.get_template(), data)
    
    #===========================================================================
    # Message Helpers
    #===========================================================================
    
    def message_debug(self, message):
        debug(self.request, message)
        
    def message_info(self, message):
        info(self.request, message)
        
    def message_success(self, message):
        success(self.request, message)
        
    def message_warning(self, message):
        warning(self.request, message)
        
    def message_error(self, message):
        error(self.request, message)
        
    def add_message(self, msgtype, message):
        add_message(self.request, msgtype, message)
    
    
class WarningProxy(object):
    __stuff = {}
    def __init__(self, logger, obj):
        WarningProxy.__stuff[self] = {}
        WarningProxy.__stuff[self]['logger'] = logger
        WarningProxy.__stuff[self]['obj'] = obj

    def __getattr__(self, attr):
        WarningProxy.__stuff[self]['logger'].lock()
        val = getattr(WarningProxy.__stuff[self]['obj'], attr)
        WarningProxy.__stuff[self]['logger'].release()
        return WarningProxy(WarningProxy.__stuff[self]['logger'], val)
    
    def __setattr__(self, attr, value):
        WarningProxy.__stuff[self]['logger'].lock()
        setattr(WarningProxy.__stuff[self]['obj'], attr)
        WarningProxy.__stuff[self]['logger'].release()
        
    def __call__(self, *args, **kwargs):
        WarningProxy.__stuff[self]['logger'].lock()
        val = WarningProxy.__stuff[self]['obj'](*args, **kwargs)
        WarningProxy.__stuff[self]['logger'].release()
        return val
    
    
class WarningLogger(object):
    def __init__(self):
        self.proxies = []
        self.queue = []
        self._old = warnings.showwarning
        
    def proxy(self, obj):
        return WarningProxy(self, obj)
    
    def lock(self):
        warnings.showwarning = self._showwarning
        
    def _showwarning(self, message, category, filename, lineno, fileobj=None):
        self.queue.append((message, category, filename, lineno))
        self._old(message, category, filename, lineno, fileobj)
    
    def release(self):
        warnings.showwarning = self._old
        
    def get(self):
        queue = list(self.queue)
        self.queue = []
        return queue
    
    def reset(self):
        self.queue = []
    
    
class Lazy(object):
    def __init__(self, real):
        self.__real = real
        self.__cache = {}
        
    def __getattr__(self, attr):
        if attr not in self.__cache:
            self.__cache[attr] = getattr(self.__real, attr)
        return self.__cache[attr]
    

def dequeue(limit=None):
    from mailchimp.models import Queue
    for camp in Queue.objects.dequeue(limit):
        yield camp
        
def is_queued_or_sent(object):
    from mailchimp.models import Queue, Campaign
    object_id = object.pk
    content_type = ContentType.objects.get_for_model(object)
    q = Queue.objects.filter(content_type=content_type, object_id=object_id)
    if q.count():
        return q[0]
    c = Campaign.objects.filter(content_type=content_type, object_id=object_id)
    if c.count():
        return c[0]
    return False

# this has to be down here to prevent circular imports
from mailchimp.chimp import Connection
# open a non-connected connection (lazily connect on first get_connection call)
CONNECTION = Connection(secure=SECURE)

def get_connection():
    if not CONNECTION.is_connected:
        CONNECTION.connect(API_KEY)
    return CONNECTION


from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site
from mailchimp.chimpy.chimpy import Connection as BaseConnection, ChimpyException
from mailchimp.utils import wrap, build_dict, Cache, WarningLogger
from mailchimp.exceptions import (MCCampaignDoesNotExist, MCListDoesNotExist,
    MCConnectionFailed, MCTemplateDoesNotExist)
from mailchimp.constants import *
from mailchimp.settings import WEBHOOK_KEY
import datetime


class SegmentCondition(object):
    OPERATORS = {
        'eq': lambda a,b: a == b,
        'ne': lambda a,b: a != b,
        'gt': lambda a,b: a > b,
        'lt': lambda a,b: a < b,
        'like': lambda a,b: a in b,
        'nlike': lambda a,b: a not in b,
        'starts': lambda a,b: str(a).startswith(str(b)),
        'ends': lambda a,b: str(a).endswith(str(b))
    }
    
    def __init__(self, field, op, value):
        self.field = field
        self.op = op
        self.value = value
        check_function_name = 'check_%s' % self.field
        if not hasattr(self, check_function_name):
            check_function_name = 'merge_check'
        self.checker = getattr(self, check_function_name)
        
    def check(self, member):
        return self.checker(member)
    
    def check_interests(self, member):
        interests = self.value.split(',')
        if self.op == 'all':
            for interest in interests:
                if interest not in member.interests:
                    return False
            return True
        elif self.op == 'one':
            for interest in interests:
                if interest in member.interests:
                    return True
            return False
        else:
            for interest in interests:
                if interest in member.interests:
                    return False
            return True
        
    def merge_check(self, member):
        return self.OPERATORS[self.op](member.merges[self.field.upper()], self.value)


class BaseChimpObject(object):
    _attrs = ()
    _methods = ()
    
    verbose_attr = 'id'
    cache_key = 'id'
    
    def __init__(self, master, info):
        self.master = master
        for attr in self._attrs:
            setattr(self, attr, info[attr])
            
        base = self.__class__.__name__.lower()
        self.cache = master.cache.get_child_cache(getattr(self, self.cache_key))
        self.con = master.con
        
        for method in self._methods:
            setattr(self, method, wrap(base, self.master.con, method, self.id))
            
    def __repr__(self):
        return '<%s object: %s>' % (self.__class__.__name__, getattr(self, self.verbose_attr))
        
        
class Campaign(BaseChimpObject):
    _attrs = ('archive_url', 'create_time', 'emails_sent', 'folder_id',
              'from_email', 'from_name', 'id', 'inline_css', 'list_id',
              'send_time', 'status', 'subject', 'title', 'to_email', 'type', 
              'web_id')
    
    _methods =  ('delete', 'pause', 'replicate', 'resume', 'schedule',
                 'send_now', 'send_test', 'unschedule')
    
    verbose_attr = 'subject'

    def __init__(self, master, info):
        super(Campaign, self).__init__(master, info)
        try:
            self.list = self.master.get_list_by_id(self.list_id)
        except MCListDoesNotExist:
            self.list = None
        self._content = None
        self.frozen_info = info
        
    def __unicode__(self):
        return self.subject
    __str__ = __unicode__
    
    @property
    def content(self):
        return self.get_content()

    def get_content(self):
        if self._content is None:
            self._content = self.con.campaign_content(self.id)
        return self._content
    
    def send_now_async(self):
        now = datetime.datetime.utcnow()
        soon = now + datetime.timedelta(minutes=1)
        return self.schedule(soon)

    def delete(self):
        return self.con.campaign_delete(self.id)
        
    def pause(self):
        return self.con.campaign_pause(self.id)
        
    def update(self):
        status = []
        for key, value in self._get_diff():
            status.append(self.con.campaign_update(self.id, key, value))
        return all(status)
    
    def _get_diff(self):
        diff = []
        new_frozen = {}
        for key in self._attrs:
            current = getattr(self, key)
            if self.frozen_info[key] != current:
                diff.append((key, current))
            new_frozen[key] = current
        self.frozen_info = new_frozen
        return diff
    
    @property
    def is_sent(self):
        return self.status == 'sent'
        
        
class Member(BaseChimpObject):
    _attrs = ('email', 'timestamp')
    
    _extended_attrs = ('id', 'ip_opt', 'ip_signup', 'merges', 'status')
    
    verbose_attr = 'email'
    cache_key = 'email'
    
    def __init__(self, master, info):
        super(Member, self).__init__(master, info)
        
    def __unicode__(self):
        return self.email
    __str__ = __unicode__

    def __getattr__(self, attr):
        if attr in self._extended_attrs:
            return self.info[attr]
        raise AttributeError, attr
    
    @property
    def interests(self):
        return [i.strip() for i in self.merges['INTERESTS'].split(',')]
    
    @property
    def info(self):
        return self.get_info()
            
    def get_info(self):
        return self.cache.get('list_member_info', self.con.list_member_info, self.master.id, self.email)
    
    def update(self):
        return self.con.list_update_member(self.master.id, self.email, self.merges)
    
    
class LazyMemberDict(dict):
    def __init__(self, master):
        super(LazyMemberDict, self).__init__()
        self._list = master
        
    def __getitem__(self, key):
        if key in self:
            return super(LazyMemberDict, self).__getitem__(key)
        value = self._list.get_member(key)
        self[key] = value
        return value
        
        
class List(BaseChimpObject):
    '''
    This represents a mailing list. Most of the methods (defined in _methods) are wrappers of the flat
    API found in chimpy.chimpy. As such, signatures are the same.
    '''
    _methods = ('batch_subscribe', 
                'batch_unsubscribe', 
                'subscribe', # Sig: (email_address,merge_vars{},email_type='text',double_optin=True)
                'unsubscribe')
    
    _attrs = ('id', 'member_count', 'date_created', 'name', 'web_id')
    
    verbose_attr = 'name'
    
    def __init__(self, *args, **kwargs):
        super(List, self).__init__(*args, **kwargs)
        self.members = LazyMemberDict(self)
    
    def segment_test(self, match, conditions):
        return self.master.con.campaign_segment_test(self.id, {'match': match, 'conditions': conditions})
    
    def add_interest_group(self, groupname):
        return self.master.con.list_interest_group_add(self.id, groupname)
        
    def remove_interest_group(self, groupname):
        return self.master.con.list_interest_group_del(self.id, groupname)
        
    def update_interest_group(self, oldname, newname):
        return self.master.con.list_interest_group_update(self.id, oldname, newname)
    
    def add_interests_if_not_exist(self, *interests):
        self.cache.flush('interest_groups')
        interest_groups = self.interest_groups['groups']
        for interest in set(interests):
            if interest not in interest_groups:
                self.add_interest_group(interest)
                interest_groups.append(interest)
                
    @property
    def webhooks(self):
        return self.get_webhooks()
    
    def get_webhooks(self):
        return self.cache.get('webhooks', self.master.con.list_webhooks, self.id)
    
    def add_webhook(self, url, actions, sources):
        return self.master.con.list_webhook_add(self.id, url, actions, sources)
    
    def remove_webhook(self, url):
        return self.master.con.list_webhook_del(self.id, url)
    
    def add_webhook_if_not_exists(self, url, actions, sources):
        for webhook in self.webhooks:
            if webhook['url'] == url:
                return True
        return self.add_webhook(url, actions, sources)
    
    def install_webhook(self):
        domain = Site.objects.get_current().domain
        if not (domain.startswith('http://') or domain.startswith('https://')):
            domain = 'http://%s' % domain
        if domain.endswith('/'):
            domain = domain[:-1] 
        url = domain + reverse('mailchimp_webhook', kwargs={'key': WEBHOOK_KEY})
        actions = {'subscribe': True,
                   'unsubscribe': True,
                   'profile': True,
                   'cleaned': True,
                   'upemail': True,}
        sources = {'user': True,
                   'admin': True,
                   'api': False}
        return self.add_webhook_if_not_exists(url, actions, sources)
    
    @property
    def interest_groups(self):
        return self.get_interest_groups()
    
    def get_interest_groups(self):
        return self.cache.get('interest_groups', self.master.con.list_interest_groups, self.id)
    
    def add_merge(self, key, desc, req={}):
        return self.master.con.list_merge_var_add(self.id, key, desc, req if req else False)
        
    def remove_merge(self, key):
        return self.master.con.list_merge_var_del(self.id, key)
    
    def add_merges_if_not_exists(self, *new_merges):
        self.cache.flush('merges')
        merges = [m['tag'].upper() for m in self.merges]
        for merge in set(new_merges):
            if merge.upper() not in merges:
                self.add_merge(merge, merge, False)
                merges.append(merge.upper())
    
    @property
    def merges(self):
        return self.get_merges()
    
    def get_merges(self):
        return self.cache.get('merges', self.master.con.list_merge_vars, self.id)
    
    def __unicode__(self):
        return self.name
    __str__ = __unicode__
    
    def get_member(self, email):
        try:
            data = self.master.con.list_member_info(self.id, email)
        except ChimpyException:
            return None
        # actually it would make more sense giving the member everything
        memberdata = {}
        memberdata['timestamp'] = data['timestamp']
        memberdata['email'] = data['email']
        return Member(self, memberdata)
    
    def filter_members(self, segment_opts):
        """
        segment_opts = {'match': 'all' if self.segment_options_all else 'any',
        'conditions': simplejson.loads(self.segment_options_conditions)}
        """
        mode = all if segment_opts['match'] == 'all' else any
        conditions = [SegmentCondition(**dict((str(k), v) for k,v in c.items())) for c in segment_opts['conditions']]
        for email, member in self.members.items():
            if mode([condition.check(member) for condition in conditions]):
                yield member
    
    
class Template(BaseChimpObject):
    _attrs = ('id', 'layout', 'name', 'preview_image', 'sections')
    
    verbose_attr = 'name'
    
    def build(self, **kwargs):
        class BuiltTemplate(object):
            def __init__(self, template, data):
                self.template = template
                self.data = data
                self.id = self.template.id
            
            def __iter__(self):
                return iter(self.data.items())
        data = {}
        for key, value in kwargs.items():
            if key in self.sections:
                data['html_%s' % key] = value
        return BuiltTemplate(self, data)


class Connection(object):
    REGULAR = REGULAR_CAMPAIGN
    PLAINTEXT = PLAINTEXT_CAMPAIGN
    ABSPLIT = ABSPLIT_CAMPAIGN
    RSS = RSS_CAMPAIGN
    TRANS = TRANS_CAMPAIGN
    AUTO = AUTO_CAMPAIGN
    DOES_NOT_EXIST = {
        'templates': MCTemplateDoesNotExist,
        'campaigns': MCCampaignDoesNotExist,
        'lists': MCListDoesNotExist,
    }
    
    def __init__(self, api_key=None, secure=False, check=True):
        self._secure = secure
        self._check = check
        self._api_key = None
        self.con = None
        self.is_connected = False
        if api_key is not None:
            self.connect(api_key)
            
    def connect(self, api_key):
        self._api_key = api_key
        self.cache = Cache(api_key)
        self.warnings = WarningLogger()
        self.con = self.warnings.proxy(BaseConnection(self._api_key, self._secure))
        if self._check:
            status = self.ping()
            if status != STATUS_OK:
                raise MCConnectionFailed(status)
        self.is_connected = True
        
    def ping(self):
        return self.con.ping()
        
    @property
    def campaigns(self):
        return self.get_campaigns()
    
    def get_campaigns(self):
        return self.cache.get('campaigns', self._get_categories)
    
    @property
    def lists(self):
        return self.get_lists()
    
    def get_lists(self):
        return self.cache.get('lists', self._get_lists)
    
    @property
    def templates(self):
        return self.get_templates()
    
    def get_templates(self):
        return self.cache.get('templates', self._get_templates)
    
    def _get_categories(self):
        return build_dict(self, Campaign, self.con.campaigns())
    
    def _get_lists(self):
        return build_dict(self, List, self.con.lists())
    
    def _get_templates(self):
        return build_dict(self, Template, self.con.campaign_templates())
    
    def get_list_by_id(self, id):
        return self._get_by_id('lists', id)
    
    def get_campaign_by_id(self, id):
        return self._get_by_id('campaigns', id)
            
    def get_template_by_id(self, id):
        return self._get_by_id('templates', id)
    
    def get_template_by_name(self, name):
        return self._get_by_key('templates', 'name', name)
        
    def _get_by_id(self, thing, id):
        try:
            return getattr(self, thing)[id]
        except KeyError:
            self.cache.flush(thing)
            try:
                return getattr(self, thing)[id]
            except KeyError:
                raise self.DOES_NOT_EXIST[thing](id)
            
    def _get_by_key(self, thing, name, key):
        for id, obj in getattr(self, thing).items():
            if getattr(obj, name) == key:
                return obj
        raise self.DOES_NOT_EXIST[thing]('%s=%s' % (name, key))
        
    def create_campaign(self, campaign_type, campaign_list, template, subject,
            from_email, from_name, to_email, folder_id=None,
            tracking={'opens':True, 'html_clicks': True}, title='',
            authenticate=False, analytics={}, auto_footer=False,
            generate_text=False, auto_tweet=False, segment_opts={},
            type_opts={}):
        """
        Creates a new campaign and returns it for the arguments given.
        """
        options = {}
        if title:
            options['title'] = title
        else:
            options['title'] = subject
        options['list_id'] = campaign_list.id
        options['template_id'] = template.id
        options['subject'] = subject
        options['from_email'] = from_email
        options['from_name'] = from_name
        options['to_email'] = to_email
        if folder_id:
            options['folder_id'] = folder_id
        options['tracking'] = tracking
        options['authenticate'] = bool(authenticate)
        if analytics:
            options['analytics'] = analytics
        options['auto_footer'] = bool(auto_footer)
        options['generate_text'] = bool(generate_text)
        options['auto_tweet'] = bool(auto_tweet)
        content = dict(template)
        kwargs = {}
        if segment_opts.get('conditions', None):
            kwargs['segment_opts'] = segment_opts
        if type_opts:
            kwargs['type_opts'] = type_opts
        cid = self.con.campaign_create(campaign_type, options, content,
            **kwargs)
        camp = self.get_campaign_by_id(cid)
        camp.template_object = template
        return camp
    
    def queue(self, campaign_type, contents, list_id, template_id, subject,
        from_email, from_name, to_email, folder_id=None, tracking_opens=True,
        tracking_html_clicks=True, tracking_text_clicks=False, title=None,
        authenticate=False, google_analytics=None, auto_footer=False,
        auto_tweet=False, segment_options=False, segment_options_all=True,
        segment_options_conditions=[], type_opts={}, obj=None):
        from mailchimp.models import Queue
        kwargs = locals().copy()
        del kwargs['Queue']
        del kwargs['self']
        return Queue.objects.queue(**kwargs)



from django.core.management import BaseCommand
from mailchimp.utils import get_connection


class Command(BaseCommand):
    def handle(self, *args, **options):
        print 'Installing webhooks for all lists'
        c = get_connection()
        for list in c.lists.values():
            print 'Checking list %s' % list.name
            # def add_webhook_if_not_exists(self, url, actions, sources):
            if list.install_webhook():
                print '  ok'
            else:
                print '  ERROR!'
        print 'Done'


from django.conf.urls.defaults import *
from mailchimp.settings import VIEWS_INFO, VIEWS_OVERVIEW, VIEWS_SCHEDULE_OBJECT, VIEWS_TEST_OBJECT
from mailchimp.views import webhook, dequeue, cancel, test_real

urlpatterns = patterns('',
    url(r'^$', VIEWS_OVERVIEW, name='mailchimp_overview', kwargs={'page':'1'}),
    url(r'^(?P<page>\d+)/$', VIEWS_OVERVIEW, name='mailchimp_overview'),
    url(r'^send/(?P<content_type>\d+)/(?P<pk>\d+)/$', VIEWS_SCHEDULE_OBJECT, name='mailchimp_schedule_for_object'),
    url(r'^test/(?P<content_type>\d+)/(?P<pk>\d+)/$', VIEWS_TEST_OBJECT, name='mailchimp_test_for_object'),
    url(r'^test/(?P<content_type>\d+)/(?P<pk>\d+)/real/$', test_real, name='mailchimp_real_test_for_object'),
    url(r'^info/(?P<campaign_id>\w+)/$', VIEWS_INFO, name='mailchimp_campaign_info'),
    url(r'^dequeue/(?P<id>\d+)/', dequeue, name='mailchimp_dequeue'),
    url(r'^cancel/(?P<id>\d+)/', cancel, name='mailchimp_cancel'),
    url(r'^webhook/(?P<key>\w+)/', webhook, name='mailchimp_webhook'),
)



from django.dispatch import Signal

args = ["list", "fired_at", "email", "interests", "fname", "lname", "merges"]

mc_subscribe = Signal(providing_args=args)
mc_unsubscribe = Signal(providing_args=args)
mc_profile = Signal(providing_args=args)
mc_upemail = Signal(providing_args=["list", "old_email", "new_email", "fired_at"])
mc_cleaned = Signal(providing_args=["fired_at", "list", "reason", "email"])


def get_signal(name):
    return globals()['mc_%s' % name]


"""
Tests for chimpy. Run them with noserunner

You need to activate groups in the Mailchimp web UI before running tests:

 * Browse to http://admin.mailchimp.com
 * List setting -> Groups for segmentation 
 * Check "add groups to my list"

"""

import os
import pprint
import operator
import random
import md5
import datetime

import chimpy

chimp = None


EMAIL_ADDRESS = 'casualbear@googlemail.com'
EMAIL_ADDRESS2 = 'dummy@dummy.com'
LIST_NAME = 'unittests'
LIST_ID = None


def setup_module():
    assert 'MAILCHIMP_APIKEY' in os.environ, \
        "please set the MAILCHIMP_APIKEY environment variable\n" \
        "you can get a new api key by calling:\n" \
        " wget 'http://api.mailchimp.com/1.1/?output=json&method=login" \
        "&password=xxxxxx&username=yyyyyyyy' -O apikey"


    global chimp
    chimp = chimpy.Connection(os.environ['MAILCHIMP_APIKEY'])


def test_ping():
    assert chimp.ping() == "Everything's Chimpy!"


def test_lists():
    lists = chimp.lists()
    pprint.pprint(lists)
    list_names = map(lambda x: x['name'], lists)
    assert LIST_NAME in list_names


def list_id():
    global LIST_ID
    if LIST_ID is None:
        test_list = [x for x in chimp.lists() if x['name'] == LIST_NAME].pop()
        LIST_ID = test_list['id']
    return LIST_ID

# use double_optin=False to prevent manual intervention
def test_list_subscribe_and_unsubscribe():
    result = chimp.list_subscribe(list_id(), EMAIL_ADDRESS,
                                    {'FIRST': 'unit', 'LAST': 'tests'},
                                    double_optin=False)
    pprint.pprint(result)
    assert result == True

    members = chimp.list_members(list_id())
    emails = map(lambda x: x['email'], members)
    print members
    assert EMAIL_ADDRESS in emails

    result = chimp.list_unsubscribe(list_id(),
                                    EMAIL_ADDRESS,
                                    delete_member=True,
                                    send_goodbye=False,
                                    send_notify=False)
    pprint.pprint(result)
    assert result == True

def test_list_batch_subscribe_and_batch_unsubscribe():
    batch = [{'EMAIL':EMAIL_ADDRESS,'EMAIL_TYPE':'html'},
             {'EMAIL':EMAIL_ADDRESS2,'EMAIL_TYPE':'text'}]

    result = chimp.list_batch_subscribe(list_id(),
                                        batch,
                                        double_optin=False,
                                        update_existing=False,
                                        replace_interests=False)
    
    assert result['success_count'] == 2

    members = chimp.list_members(list_id())
    emails = map(lambda x: x['email'], members)
    assert EMAIL_ADDRESS in emails
    assert EMAIL_ADDRESS2 in emails

    result = chimp.list_batch_unsubscribe(list_id(),
                                          [EMAIL_ADDRESS,EMAIL_ADDRESS2],
                                          delete_member=True,
                                          send_goodbye=False,
                                          send_notify=False)

    assert result['success_count'] == 2

def test_list_interest_groups_add_and_delete():
    # check no lists exists
    pprint.pprint(chimp.list_interest_groups(list_id()))
    assert len(chimp.list_interest_groups(list_id())['groups']) == 0

    # add list
    assert chimp.list_interest_group_add(list_id(), 'test')
    assert len(chimp.list_interest_groups(list_id())['groups']) == 1

    # delete list
    assert chimp.list_interest_group_del(list_id(), 'test')
    assert len(chimp.list_interest_groups(list_id())['groups']) == 0

def test_list_merge_vars_add_and_delete():
    pprint.pprint(chimp.list_merge_vars(list_id()))
    assert len(chimp.list_merge_vars(list_id())) == 3

    # add list
    assert chimp.list_merge_var_add(list_id(), 'test', 'some text')
    assert len(chimp.list_merge_vars(list_id())) == 4

    # delete list
    assert chimp.list_merge_var_del(list_id(), 'test')
    assert len(chimp.list_merge_vars(list_id())) == 3

def test_list_update_member_and_member_info():
    # set up
    assert chimp.list_subscribe(list_id(), EMAIL_ADDRESS,
                                    {'FIRST': 'unit', 'LAST': 'tests'},
                                    double_optin=False)
    assert chimp.list_merge_var_add(list_id(), 'TEST', 'test merge var')
    assert chimp.list_interest_group_add(list_id(), 'tlist')


    # update member and get the info back
    assert chimp.list_update_member(list_id(), EMAIL_ADDRESS,
                                    {'TEST': 'abc',
                                    'INTERESTS': 'tlist'}, replace_interests=False)
    info = chimp.list_member_info(list_id(), EMAIL_ADDRESS)
    pprint.pprint(info)

    # tear down
    assert chimp.list_merge_var_del(list_id(), 'TEST')
    assert chimp.list_interest_group_del(list_id(), 'tlist')
    assert chimp.list_unsubscribe(list_id(), EMAIL_ADDRESS,
                                    delete_member=True,
                                    send_goodbye=False,
                                    send_notify=False)

    # check the info matches the set up
    assert 'TEST' in info['merges']
    assert info['merges']['TEST'] == 'abc'
    assert 'tlist' in info['merges']['INTERESTS']


def test_create_delete_campaign():
    uid = md5.new(str(random.random())).hexdigest()
    subject = 'chimpy campaign test %s' % uid
    options = {'list_id': list_id(),
           'subject': subject,
           'from_email': EMAIL_ADDRESS,
           'from_name': 'chimpy',
           'generate_text': True
           }

    #this just to be sure flatten utility is working
    segment_opts = {'match': 'any', 
            'conditions':[{'field': 'date', 'op': 'gt', 'value': '2000-01-01'},
                          {'field': 'email', 'op': 'like', 'value': '@'}]}

    html = """ <html><body><h1>My test newsletter</h1><p>Just testing</p>
               <a href="*|UNSUB|*">Unsubscribe</a>*|REWARDS|*</body>"""


    content = {'html': html}
    cid = chimp.campaign_create('regular', options, content, segment_opts=segment_opts)

    assert isinstance(cid, str)

    # check if the new campaign really is there
    campaigns = chimp.campaigns(filter_subject=subject)
    assert len(campaigns)==1
    assert campaigns[0]['id'] == cid

    # our content properly addd?
    final_content = chimp.campaign_content(cid)
    assert '<h1>My test newsletter</h1>' in final_content['html']
    assert 'My test newsletter' in final_content['text']

    # clean up
    chimp.campaign_delete(cid)

def test_replicate_update_campaign():
    """ replicates and updates a campaign """

    uid = md5.new(str(random.random())).hexdigest()
    subject = 'chimpy campaign test %s' % uid
    options = {'list_id': list_id(),
           'subject': subject,
           'from_email': EMAIL_ADDRESS,
           'from_name': 'chimpy',
           'generate_text': True
           }

    html = """ <html><body><h1>My test newsletter</h1><p>Just testing</p>
               <a href="*|UNSUB|*">Unsubscribe</a>*|REWARDS|*</body>"""


    content = {'html': html}
    cid = chimp.campaign_create('regular', options, content)

    newcid = chimp.campaign_replicate(cid=cid)
    assert isinstance(newcid, str)

    newsubject = 'Fresh subject ' + uid
    newtitle = 'Custom title ' + uid

    res = chimp.campaign_update(newcid, 'subject', newsubject)
    assert res is True
    res = chimp.campaign_update(newcid, 'title', newtitle)
    assert res is True

    campaigns = chimp.campaigns(filter_subject=newsubject)
    assert len(campaigns)==1
    campaigns = chimp.campaigns(filter_title=newtitle)
    assert len(campaigns)==1

    #clean up
    chimp.campaign_delete(newcid)
    chimp.campaign_delete(cid)

def test_schedule_campaign():
    """ schedules and unschedules a campaign """

    uid = md5.new(str(random.random())).hexdigest()
    subject = 'chimpy campaign schedule test %s' % uid
    options = {'list_id': list_id(),
           'subject': subject,
           'from_email': EMAIL_ADDRESS,
           'from_name': 'chimpy',
           'generate_text': True
           }

    html = """ <html><body><h1>My test newsletter</h1><p>Just testing</p>
               <a href="*|UNSUB|*">Unsubscribe</a>*|REWARDS|*</body>"""


    content = {'html': html}
    cid = chimp.campaign_create('regular', options, content)

    schedule_time = datetime.datetime(2012, 12, 20, 19, 0, 0)
    chimp.campaign_schedule(cid, schedule_time)

    campaign = chimp.campaigns(filter_subject=subject)[0]
    assert campaign['status'] == 'schedule'
    assert campaign['send_time'] == 'Dec 20, 2012 07:00 pm'

    chimp.campaign_unschedule(cid)
    campaign = chimp.campaigns(filter_subject=subject)[0]
    assert campaign['status'] == 'save'

    #clean up
    chimp.campaign_delete(cid)

def test_rss_campaign():
    """ add, pause, resume rss campaign """

    uid = md5.new(str(random.random())).hexdigest()
    subject = 'chimpy campaign rss test %s' % uid
    options = {'list_id': list_id(),
           'subject': subject,
           'from_email': EMAIL_ADDRESS,
           'from_name': 'chimpy',
           'generate_text': True
           }

    html = """ <html><body><h1>My test RSS newsletter</h1><p>Just testing</p>
               <a href="*|UNSUB|*">Unsubscribe</a>*|REWARDS|*</body>"""


    content = {'html': html}
    type_opts = {'url': 'http://mailchimp.com/blog/rss'}

    cid = chimp.campaign_create('rss', options, content, type_opts=type_opts)
    campaign = chimp.campaigns(filter_subject=subject)[0]
    assert campaign['type'] == 'rss'

    # Todo: Could not find a way to activate the RSS from the API. You need to
    # activate before being able to test pause and resume. send_now and schedule
    # didn't do the trick.

    #chimp.campaign_pause(cid)
    #chimp.campaign_resume(cid)

    #clean up
    chimp.campaign_delete(cid)




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding model 'Queue'
        db.create_table('mailchimp_queue', (
            ('type_opts', self.gf('django.db.models.fields.TextField')()),
            ('segment_options_all', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('contents', self.gf('django.db.models.fields.TextField')()),
            ('subject', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('campaign_type', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('authenticate', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('from_email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('segment_options', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('list_id', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('auto_tweet', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('from_name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('folder_id', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('generate_text', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('to_email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('tracking_text_clicks', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('auto_footer', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('tracking_html_clicks', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('google_analytics', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('segment_options_conditions', self.gf('django.db.models.fields.TextField')()),
            ('template_id', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('tracking_opens', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
        ))
        db.send_create_signal('mailchimp', ['Queue'])
    
    
    def backwards(self, orm):
        
        # Deleting model 'Queue'
        db.delete_table('mailchimp_queue')
    
    
    models = {
        'mailchimp.campaign': {
            'Meta': {'object_name': 'Campaign'},
            'campaign_id': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sent_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'mailchimp.queue': {
            'Meta': {'object_name': 'Queue'},
            'authenticate': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'auto_footer': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'auto_tweet': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'campaign_type': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'contents': ('django.db.models.fields.TextField', [], {}),
            'folder_id': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'from_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'from_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'generate_text': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'google_analytics': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'list_id': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'segment_options': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'segment_options_all': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'segment_options_conditions': ('django.db.models.fields.TextField', [], {}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'template_id': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'to_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'tracking_html_clicks': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'tracking_opens': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'tracking_text_clicks': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'type_opts': ('django.db.models.fields.TextField', [], {})
        },
        'mailchimp.reciever': {
            'Meta': {'object_name': 'Reciever'},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'recievers'", 'to': "orm['mailchimp.Campaign']"}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['mailchimp']



from django.core.management import BaseCommand
from django.contrib.sites.models import Site
from mailchimp.utils import get_connection


class Command(BaseCommand):
    def handle(self, *args, **options):
        print 'Installing site segment groups for all lists and all sites'
        c = get_connection()
        interests = []
        for site in Site.objects.all():
            interests.append(site.domain)
        for list in c.lists.values():
            print 'Checking list %s' % list.name
            list.add_interests_if_not_exist(*interests)
            print '  ok'
        print 'Done'


from django.core.management import BaseCommand
from mailchimp.utils import dequeue
from optparse import make_option


class Command(BaseCommand):
    
    def handle(self, *args, **options):
        if len(args) and args[0].isdigit():
            limit = int(args[0])
        else:
            limit = None
        print 'Dequeueing Campaigns'
        done = False
        for camp in dequeue(limit):
            done = True
            if camp:
                print '- Dequeued campaign %s (%s)' % (camp.name, camp.campaign_id)
            else:
                print 'ERROR'
        if not done:
            print 'Nothing to dequeue'
        print 'Done'


from setuptools import setup, find_packages

version = __import__('mailchimp').__version__

setup(
    name = 'django-mailchimp',
    version = version,
    description = 'Mailchimp wrapper for Django',
    author = 'Jonas Obrist',
    url = 'http://github.com/ojii/django-mailchimp',
    packages = find_packages(),
    zip_safe=False,
    package_data={
        'mailchimp': [
            'templates/mailchimp/*.html',
            'locale/en/LC_MESSAGES/django.po',
            'locale/en/LC_MESSAGES/django.mo',
            'locale/de/LC_MESSAGES/django.po',
            'locale/de/LC_MESSAGES/django.mo',
        ],
    },
)


import urllib
import urllib2
import pprint
from utils import transform_datetime
from utils import flatten
from warnings import warn
from django.utils import simplejson
_debug = 1


class ChimpyException(Exception):
    pass

class ChimpyWarning(Warning):
    pass


class Connection(object):
    """mailchimp api connection"""

    output = "json"
    version = '1.2'

    def __init__(self, apikey=None, secure=False):
        self._apikey = apikey
        proto = 'http'
        if secure:
            proto = 'https'
        api_host = 'api.mailchimp.com'
        if '-' in apikey:
            key, dc = apikey.split('-')
        else:
            dc = 'us1'
        api_host = dc + '.' + api_host

        self.url = '%s://%s/%s/' % (proto, api_host, self.version)
        self.opener = urllib2.build_opener()
        self.opener.addheaders = [('Content-Type', 'application/x-www-form-urlencoded')]
        
    def _rpc(self, method, **params):
        """make an rpc call to the server"""

        params = urllib.urlencode(params, doseq=True)

        if _debug > 1:
            print __name__, "making request with parameters"
            pprint.pprint(params)
            print __name__, "encoded parameters:", params

        response = self.opener.open("%s?method=%s" %(self.url, method), params)
        data = response.read()
        response.close()

        if _debug > 1:
            print __name__, "rpc call received", data

        result = simplejson.loads(data)

        try:
            if 'error' in result:
                raise ChimpyException("%s:\n%s" % (result['error'], params))
        except TypeError:
            # thrown when results is not iterable (eg bool)
            pass

        return result

    def _api_call(self, method, **params):
        """make an api call"""


        # flatten dict variables
        params = dict([(str(k), v.encode('utf-8') if isinstance(v, unicode) else v) for k,v in flatten(params).items()])
        params['output'] = self.output
        params['apikey'] = self._apikey

        return self._rpc(method=method, **params)

    def ping(self):
        return self._api_call(method='ping')

    def lists(self):
        return self._api_call(method='lists')

    def list_batch_subscribe(self,
                             id,
                             batch,
                             double_optin=True,
                             update_existing=False,
                             replace_interests=False):

        return self._api_call(method='listBatchSubscribe',
                              id=id,
                              batch=batch,
                              double_optin=double_optin,
                              update_existing=update_existing,
                              replace_interests=replace_interests)

    def list_batch_unsubscribe(self,
                               id,
                               emails,
                               delete_member=False,
                               send_goodbye=True,
                               send_notify=False):

        return self._api_call(method='listBatchUnsubscribe',
                              id=id,
                              emails=emails,
                              delete_member=delete_member,
                              send_goodbye=send_goodbye,
                              send_notify=send_notify)

    def list_subscribe(self,
                       id,
                       email_address,
                       merge_vars,
                       email_type='text',
                        double_optin=True):
        return self._api_call(method='listSubscribe',
                              id=id,
                              email_address=email_address,
                              merge_vars=merge_vars,
                              email_type=email_type,
                              double_optin=double_optin)

    def list_unsubscribe(self,
                         id,
                         email_address,
                         delete_member=False,
                         send_goodbye=True,
                         send_notify=True):
        return self._api_call(method='listUnsubscribe',
                              id=id,
                              email_address=email_address,
                              delete_member=delete_member,
                              send_goodbye=send_goodbye,
                              send_notify=send_notify)

    def list_update_member(self,
                           id,
                           email_address,
                           merge_vars,
                           email_type='',
                           replace_interests=True):
        return self._api_call(method='listUpdateMember',
                              id=id,
                              email_address=email_address,
                              merge_vars=merge_vars,
                              email_type=email_type,
                              replace_interests=replace_interests)

    def list_member_info(self, id, email_address):
        return self._api_call(method='listMemberInfo',
                              id=id,
                              email_address=email_address)

    def list_members(self, id, status='subscribed', since=None, start=0, limit=100):
        return self._api_call(method='listMembers', id=id, status=status, since=since, start=start, limit=limit)

    def list_interest_groups(self, id):
        return self._api_call(method='listInterestGroups', id=id)

    def list_interest_group_add(self, id, name):
        return self._api_call(method='listInterestGroupAdd', id=id, group_name=name)

    def list_interest_group_del(self, id, name):
        return self._api_call(method='listInterestGroupDel', id=id, group_name=name)

    def list_merge_vars(self, id):
        return self._api_call(method='listMergeVars', id=id)

    def list_merge_var_add(self, id, tag, name, req=False):
        return self._api_call(method='listMergeVarAdd', id=id, tag=tag, name=name, req=req)

    def list_merge_var_del(self, id, tag):
        return self._api_call(method='listMergeVarDel', id=id, tag=tag)
    
    def list_webhooks(self, id):
        return self._api_call(method='listWebhooks', id=id)
    
    # public static listWebhookAdd(string apikey, string id, string url, array actions, array sources)
    def list_webhook_add(self, id, url, actions, sources):
        return self._api_call(method='listWebhookAdd', id=id, url=url, actions=actions, sources=sources)
    
    def list_webhook_del(self, id, url):
        return self._api_call(method='listWebhookDel', id=id, url=url)

    def campaign_content(self, cid):
        """Get the content (both html and text) for a campaign, exactly as it would appear in the campaign archive
        http://www.mailchimp.com/api/1.1/campaigncontent.func.php
        """

        return self._api_call(method='campaignContent', cid=cid)

    def campaign_create(self, campaign_type, options, content, **kwargs):
        """Create a new draft campaign to send.
        http://www.mailchimp.com/api/1.1/campaigncreate.func.php

        Optional parameters: segment_opts, type_opts
        """
        # enforce the 100 char limit (urlencoded!!!)
        title = options.get('title', options['subject'])
        if isinstance(title, unicode):
            title = title.encode('utf-8')
        titlelen = len(urllib.quote_plus(title))
        if titlelen > 99:
            title = title[:-(titlelen - 96)] + '...'
            warn("cropped campaign title to fit the 100 character limit, new title: '%s'" % title, ChimpyWarning)
        subject = options['subject']
        if isinstance(subject, unicode):
            subject = subject.encode('utf-8')
        subjlen = len(urllib.quote_plus(subject))
        if subjlen > 99:
            subject = subject[:-(subjlen - 96)] + '...'
            warn("cropped campaign subject to fit the 100 character limit, new subject: '%s'" % subject, ChimpyWarning)
        options['title'] = title
        options['subject'] = subject
        return self._api_call(method='campaignCreate', type=campaign_type, options=options, content=content, **kwargs)

    def campaign_delete(self, cid):
        """Delete a campaign.
        http://www.mailchimp.com/api/1.1/campaigndelete.func.php
        """

        return self._api_call(method='campaignDelete', cid=cid)

    def campaign_folders(self):
        """List all the folders for a user account.
        http://www.mailchimp.com/api/1.1/campaignfolders.func.php
        """

        return self._api_call(method='campaignFolders')

    def campaign_pause(self, cid):
        """Pause a RSS campaign from sending.
        http://www.mailchimp.com/api/1.1/campaignpause.func.php
        """

        return self._api_call(method='campaignPause', cid=cid)

    def campaign_replicate(self, cid):
        """Replicate a campaign.
        http://www.mailchimp.com/api/1.1/campaignreplicate.func.php
        """

        return self._api_call(method='campaignReplicate', cid=cid)

    def campaign_resume(self, cid):
        """Resume sending a RSS campaign.
        http://www.mailchimp.com/api/1.1/campaignresume.func.php
        """

        return self._api_call(method='campaignResume', cid=cid)

    def campaign_schedule(self, cid, schedule_time, schedule_time_b=None):
        """Schedule a campaign to be sent in the future.
        http://www.mailchimp.com/api/1.1/campaignschedule.func.php
        """

        schedule_time = transform_datetime(schedule_time)

        if schedule_time_b:
            schedule_time_b = transform_datetime(schedule_time_b)

        return self._api_call(method='campaignSchedule', cid=cid, schedule_time=schedule_time, schedule_time_b=schedule_time_b)

    def campaign_send_now(self, cid):
        """Send a given campaign immediately.
        http://www.mailchimp.com/api/1.1/campaignsendnow.func.php
        """

        return self._api_call(method='campaignSendNow', cid=cid)

    def campaign_send_test(self, cid, test_emails, **kwargs):
        """Send a test of this campaign to the provided email address.
        Optional parameter: send_type
        http://www.mailchimp.com/api/1.1/campaignsendtest.func.php
        """

        if isinstance(test_emails, str):
            test_emails = [test_emails]

        return self._api_call(method='campaignSendTest', cid=cid, test_emails=test_emails, **kwargs)

    def campaign_templates(self):
        """ Retrieve all templates defined for your user account """

        return self._api_call(method='campaignTemplates')

    def campaign_unschedule(self, cid):
        """Unschedule a campaign that is scheduled to be sent in the future  """

        return self._api_call(method='campaignUnschedule', cid=cid)

    def campaign_update(self, cid, name, value):
        """Update just about any setting for a campaign that has not been sent.
        http://www.mailchimp.com/api/1.1/campaignupdate.func.php
        """

        return self._api_call(method='campaignUpdate', cid=cid, name=name, value=value)

    def campaigns(self, filter_id='', filter_folder=None, filter_fromname='', filter_fromemail='',
                  filter_title='', filter_subject='', filter_sendtimestart=None, filter_sendtimeend=None,
                  filter_exact=False, start=0, limit=50):
        """Get the list of campaigns and their details matching the specified filters.
        Timestamps should be passed as datatime objects.

        http://www.mailchimp.com/api/1.1/campaigns.func.php
        """

        filter_sendtimestart = transform_datetime(filter_sendtimestart)
        filter_sendtimeend = transform_datetime(filter_sendtimeend)


        return self._api_call(method='campaigns',
                              filter_id=filter_id, filter_folder=filter_folder, filter_fromname=filter_fromname,
                              filter_fromemail=filter_fromemail, filter_title=filter_title, filter_subject=filter_subject,
                              filter_sendtimestart=filter_sendtimestart, filter_sendtimeend=filter_sendtimeend,
                              filter_exact=filter_exact, start=start, limit=limit)

    def campaign_segment_test(self, list_id, options):
        return self._api_call(method='campaignSegmentTest', list_id=list_id, options=options)